#!/usr/bin/env python

__cvsversion__ = '$Id: Notify.py,v 1.2 2007/12/14 17:33:00 jay Exp $'
__author__ = 'Jay Deiman; jay@splitstreams.com'
__copyright__ = 'Copyright 2007 Jason Deiman'
__license__ = 'GPLv3'
__all__ = ['Notify' , 'NotifyError']

import time , socket , utils
from email import MIMEText

class NotifyError(Exception):
    pass

class Notify(object):
    """
    Class to provide email notification on events, such as bans
    """
    def __init__(self , config):
        """
        Initialize the config variable
        """
        self._config = config
    
    def notifyBan(self , banIp , accesses , prevBanCount , perm=False):
        """
        Send an email notifying a config specified recipient of
        an IP ban
        """
        # First build the body of the message
        if utils.DEBUG:
            print 'notifyBan() called for %s' % banIp
        msgBody = 'DictDefence Banned an IP: %s\n\n' % banIp
        msgBody += 'Number of Previous Bans: %s\n' % prevBanCount
        msgBody += 'Permanently Banned: %s\n\n' % (('No' , 'Yes')[perm])
        msgBody += 'Accesses:\n'
        msgBody += '%-27s%-18s%s\n' % ('Time' , 'IP' , 'Logline')
        msgBody += '%-27s%-18s%s\n' % ('-' * 24 , '-' * 15 , '-' * 36)
        for row in accesses:
            msgBody += '%-27s%-18s%s\n' % (time.ctime(row['ts']) , 
                                         row['ip_str'] ,
                                         row['logline'])
        # And now create the mime message
        sender = 'noreply@%s' % (utils.getHostName())
        msg = MIMEText.MIMEText(msgBody)
        msg['To'] = self._config.NotifyEmail
        msg['Subject'] = 'DictDefence banned: %s' % banIp
        msg['From'] = sender
        # Now we send it using our defined method
        if self._config.NotifyType == 'sendmail':
            if utils.DEBUG:
                print 'Notifying with sendmail'
            if not self._sendMsgWithSendmail(msg , sender):
                raise NotifyError , 'An error occured sending a notify' + \
                        ' email with sendmail'
        else:
            if utils.DEBUG:
                print 'Notifying with SMTP'
            if not self._sendMsgWithSmtp(msg , sender):
                raise NotifyError , 'An error occured sending a notify' + \
                        ' email with SMTP'
            
    def _sendMsgWithSendmail(self , msg , sender):
        """
        Send an email using the local sendmail program
        """
        cmd = '%s -i -f %s -- %s' % (self._config.SendmailPath , sender ,
                                     self._config.NotifyEmail)
        import os
        smh = os.popen(cmd , 'w')
        smh.write(msg.as_string())
        if smh.close() > 0:
            return False
        return True
    
    def _sendMsgWithSmtp(self , msg , sender):
        """
        Send an email using SMTP
        """
        import smtplib
        s = smtplib.SMTP()
        if utils.DEBUG:
            print 'Starting SMTP transaction'
        resp = s.connect(self._config.SmtpHost , self._config.SmtpPort)
        if utils.DEBUG:
            print resp
        if not self._isOk(resp[0]):
            s.close()
            return false
        if self._config.SmtpUseTLS:
            resp = s.starttls()
            if utils.DEBUG:
                print resp
            if not self._isOk(resp[0]):
                s.close()
                return False
        if self._config.SmtpUser:
            resp = s.login(self._config.SmtpUser , self._config.SmtpPass)
            if utils.DEBUG:
                print resp
            if not self._isOk(resp[0]):
                s.close()
                return False
        resp = s.sendmail(sender , [self._config.NotifyEmail] , msg.as_string())
        if utils.DEBUG:
            print resp
        s.close()
        return True
    
    def _isOk(self , smtpCode):
        """
        Given a number, this checks to see if it is a 2XX code, which is 
        an SMTP ok
        """
        smtpCode = int(smtpCode)
        return smtpCode >= 200 and smtpCode < 300