#!/usr/bin/env python

__cvsversion__ = '$Id: utils.py,v 1.9 2007/12/14 17:33:00 jay Exp $'
__author__ = 'Jay Deiman; jay@splitstreams.com'
__copyright__ = 'Copyright 2007 Jason Deiman'
__license__ = 'GPLv3'
__all__ = ['ip2int' , 'int2ip' , 'getBCAddrInt' , 'getBCAddrIP' , 'slashHash' ,
           'getNetworkInt' , 'getNetworkIP' , 'slash2intip' , 'isInNetwork' ,
           'int32' , 'daemonize'
]

import os , sys , socket , struct , re

DEBUG = False

def daemonize(stdin='/dev/null' , stdout='/dev/null' , stderr='/dev/null'):
    if os.fork() > 0:
        sys.exit(0)
    os.chdir('/')
    os.umask(0)
    os.setsid()
    if os.fork() > 0:
        sys.exit(0)
    
    for f in sys.stdout , sys.stderr: f.flush()
    
    si = open(stdin , 'r')
    so = open(stdout , 'a+')
    se = open(stderr , 'a+' , 0)
    os.dup2(si.fileno() , sys.stdin.fileno())
    os.dup2(so.fileno() , sys.stdout.fileno())
    os.dup2(se.fileno() , sys.stderr.fileno())
    
def ip2int(ip):
    return struct.unpack('I' , socket.inet_aton(ip))[0]

def int2ip(integer):
    return socket.inet_ntoa(struct.pack('I' , integer))

def getBCAddrInt(intip , intmask):
    global int32
    bcMask = intmask ^ int32
    return intip | bcMask

def getBCAddrIP(intip , intmask):
    return int2ip(getBCAddrInt(intip , intmask))

def getNetworkInt(intip , intmask):
    return intip & intmask

def getNetworkIP(intip , intmask):
    return int2ip(getNetworkInt(intip , intmask))

def slash2intip(slash):
    global slashHash
    return slashHash[int(slash)]

def isInNetwork(ip2check , cidrNetwork):
    networkIp , slash = cidrNetwork.split('/')
    intMask = slash2intip(slash)
    intNetworkIp = ip2int(networkIp)
    intIp2check = ip2int(ip2check)
    bcInt = getBCAddrInt(intNetworkIp , intMask)
    netInt = getNetworkInt(intNetworkIp , intMask)
    if intIp2check < netInt or intIp2check > bcInt:
        return False
    return True

def getHostName():
    fqdn = re.compile(r'^(?:[^\s\.]+\.){1,}(?:[^\s\.]+)$')
    res = socket.gethostbyaddr('127.0.0.1')
    if fqdn.match(res[0]):
        return res[0]
    for name in res[1]:
        if fqdn.match(name):
            return name

def _fillSlashHash(hash):
    global int32
    slashHash['0'] = '0'
    for i in range(32):
        tmp = int32 >> i
        dig = tmp << i
        slashHash[32 - i] = dig

slashHash = {}
int32 = 2 ** 32 -1
_fillSlashHash(slashHash)
